﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace Hucard_Transfer_Software
{
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		private void BtnSelectClick(object sender, EventArgs e)
		{

			try
			{
				if (open_rom.ShowDialog() == DialogResult.OK & open_rom.FileName != null)
				{
					Globals.romfile = open_rom.FileName;
					txt_rom.Text = Globals.romfile;
					chk_convert.Checked = false;
					chk_header.Checked = false;
					chk_region.Checked = false;
					using (var stream = new FileStream(Globals.romfile, FileMode.Open, FileAccess.Read))
					{
						if (stream.Length % 1024 == 512)
						{
							chk_header.Enabled = true;
							chk_header.Checked = true;
							lbl_header.Text = "512B Header or padded homebrew";
						}
						else
						{
							chk_header.Enabled = false;
							chk_header.Checked = false;
							lbl_header.Text = "No header";
						}

						if (stream.Length == 0x60000 | stream.Length == 0x60200)
						{
							chk_convert.Enabled = true;
							chk_convert.Checked = true;
							lbl_mirror.Text = "Rom requires mirroring";
						}
						else
						{
							chk_convert.Enabled = false;
							chk_convert.Checked = false;
							lbl_mirror.Text = "No mirroring required";
						}
						using (var br = new BinaryReader(stream))
						{
							Globals.pcerom = new byte[stream.Length];

							byte[] buffer = br.ReadBytes(Convert.ToInt32(stream.Length));
							Array.Copy(buffer, Globals.pcerom, buffer.Length);

							if (SearchRegion(Globals.pattern, buffer))
							{
								chk_region.Enabled = true;
								chk_region.Checked = true;
							}
							else
							{
								chk_region.Enabled = false;
								chk_region.Checked = false;
							}
							br.Close();
						}
						if(chk_convert.Checked == false & chk_header.Checked == false & chk_region.Checked == false)
						{
							MessageBox.Show("No modification necessary.","Clean ROM detected",MessageBoxButtons.OK,MessageBoxIcon.Information);
							Resetcontrols();
						}
						else
						{
							chk_backup.Enabled = true;
							chk_backup.Checked = true;
							btn_modify.Enabled = true;
						}

						stream.Close();
					}
				}
			}
			catch(Exception error)
			{
				MessageBox.Show("Error: " + error);
			}
		}

		private void ExitToolStripMenuItemClick(object sender, EventArgs e)
		{
			Close();
		}

		private bool SearchRegion(byte[] pattern, byte[] bytes)
		{
			bool found = false;
			int maxloop = bytes.Length - pattern.Length;

			for (int i = 0; i < maxloop; i++)
			{
				if (pattern[0] == bytes[i])
				{
					bool ismatch = true;
					for (int j = 1; j < pattern.Length; j++)
					{
						if (bytes[i + j] != pattern[j])
						{
							ismatch = false;
							break;
						}
					}
					if (ismatch)
					{
						//Check if 'NEC' is at 0x15 from the found offset
						if (bytes[i+0x0B] == 0xF0 & bytes[i+0x15] == 0x4E & bytes[i+0x16] == 0x45 & bytes[i+0x17] == 0x43)
						{
							Globals.regionlock = i+0xB;
							found = true;
							i = maxloop;
						}
						i += pattern.Length - 1;
					}
				}
			}
			return found;
		}

		private void BtnModifyClick(object sender, EventArgs e)
		{
			try
			{
				byte[] temp;
				//Remove region protection
				if (chk_region.Checked)
				{
					Globals.pcerom[Globals.regionlock] = 0x80;
				}

				// Remove the rom header
				if (chk_header.Checked)
				{
					temp = new byte[Globals.pcerom.Length];
					Array.Copy(Globals.pcerom, temp, Globals.pcerom.Length);
					Globals.pcerom = new byte[Globals.pcerom.Length - 512];
					Array.Copy(temp, 512, Globals.pcerom, 0, Globals.pcerom.Length);
				}

				//Mirror
				if (chk_convert.Checked)
				{
					temp = new byte[Globals.pcerom.Length];
					Array.Copy(Globals.pcerom, temp, Globals.pcerom.Length);
					Globals.pcerom = new byte[655360];
					Array.Copy(temp, 0, Globals.pcerom, 0, 262144);
					Array.Copy(temp, 0, Globals.pcerom, 262144, 393216);
				}

				//Make a backup
				if (chk_backup.Checked)
				{
					File.Copy(Globals.romfile, Globals.romfile + ".bak", true);
				}

				//Write the changes to the rom file
				File.WriteAllBytes(Globals.romfile,Globals.pcerom);

				if (chk_backup.Checked)
				{
					MessageBox.Show("The rom " + Globals.romfile + " was successfully modified.\r\nBackup made to: " + Globals.romfile + ".bak","Operation Successful");
				}
				else
				{
					MessageBox.Show("The rom " + Globals.romfile + " was successfully modified.","Operation Successful");
				}

			}
			catch (Exception error)
			{
				MessageBox.Show("Error: " + error);
			}

			Resetcontrols();
		}

		private void Resetcontrols()
		{
			btn_modify.Enabled = false;
			chk_backup.Enabled = false;
			chk_backup.Checked = false;
			chk_convert.Enabled = false;
			chk_convert.Checked = false;
			chk_header.Enabled = false;
			chk_header.Checked = false;
			chk_region.Enabled = false;
			chk_region.Checked = false;
			txt_rom.Text = null;
			Array.Clear(Globals.pcerom, 0, Globals.pcerom.Length);
			Globals.romfile = null;
			open_rom.FileName = null;
			Globals.regionlock = 0;
			GC.Collect();
		}

	}
}
